import os
from dataclasses import asdict, dataclass, field
from enum import Enum
from functools import cached_property

import cli
import requests

API_URL = "https://api.openai.com/v1/chat/completions"


class Role(Enum):
    user = "user"
    assistant = "assistant"


@dataclass
class Message:
    role: Role
    content: str

    def serialize(self):
        return {"role": self.role.value, "content": self.content}


@dataclass
class Messages:
    messages: list[Message] = field(default_factory=list)

    def serialize(self):
        return [message.serialize() for message in self.messages]

    def append(self, message: Message):
        self.messages.append(message)


@dataclass
class Data:
    messages: Messages
    model: str = "gpt-3.5-turbo"
    temperature: float = 1
    top_p: float = 1
    n: int = 1
    stream: bool = True

    def serialize(self):
        items = asdict(self)
        items["messages"] = self.messages.serialize()
        return items


@dataclass
class API:
    url: str = "https://api.openai.com/v1/chat/completions"

    def __post_init__(self):
        self.headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.key}",
        }

    def get_reply(self, messages: Messages):
        data = Data(messages).serialize()
        return requests.post(
            self.url, headers=self.headers, json=data, timeout=30, stream=True
        )

    @cached_property
    def key(self) -> str:
        try:
            key = cli.get("pw OPENAI_API_KEY")
        except FileNotFoundError:
            key = os.environ.get("OPENAI_API_KEY")
        if key is None:
            key = cli.prompt("OpenAI API key: ")
        return key



from dataclasses import dataclass

import cli
from rich.style import Style
from rich.text import Text

from .chatmanager import ChatManager


@dataclass
class Chat:
    personal_title: Text = Text("User: ", style=Style(color="green", bold=True))
    assistant_title: Text = Text("Chatgpt: ", style=Style(color="blue", bold=True))

    def __post_init__(self):
        self.manager = ChatManager()

    def send(self, prompt: str):
        self.manager.send(prompt)
        self.show_reply()

    def show_reply(self):
        print("")
        with cli.status(""):
            self.manager.retrieve_reply()

        cli.console.print(self.assistant_title, end="")
        for chunk in self.manager.get_reply_chunks():
            print(chunk, end="")
        print("\n")



import json
from dataclasses import dataclass

import requests

from .api import API, Message, Messages, Role


@dataclass
class ChatManager:
    messages: Messages = None
    api_response: requests.Response = None

    def __post_init__(self):
        self.messages = Messages()
        self.api = API()

    def send(self, prompt: str):
        message = Message(role=Role.user, content=prompt)
        self.messages.append(message)

    def retrieve_reply(self):
        self.api_response = self.api.get_reply(self.messages)

    def get_reply_chunks(self):
        chunks = []
        for chunk in self.get_response_chunks():
            chunks.append(chunk)
            yield chunk

        reply = "".join(chunks)
        message = Message(role=Role.assistant, content=reply)
        self.messages.append(message)

    def get_response_chunks(self):
        whitespace_seen = False
        for response in self.get_api_response_chunks():
            message = self.parse_response(response)
            if message == "\n\n" and not whitespace_seen:
                whitespace_seen = True
            elif message:
                yield message

    @classmethod
    def parse_response(cls, response_bytes: bytes):
        response_bytes_json = response_bytes.strip()
        try:
            response_info = json.loads(response_bytes_json)
        except json.JSONDecodeError:
            print(response_bytes_json)
            raise
        response = response_info["choices"][0]["delta"]
        return response.get("content")

    def get_api_response_chunks(self):
        api_response_parts = []
        for chunk in self.api_response:
            data_keyword = b"data: "
            if chunk.startswith(data_keyword) and api_response_parts:
                api_response = b"".join(api_response_parts)
                api_responses = api_response.split(data_keyword)
                for response in api_responses:
                    if response:
                        yield response
                api_response_parts = []
            api_response_parts.append(chunk)
